
-- # import Property
-- # import E0
-- # import GHC.Base
-- # import GHC.Base-Models
-- # import GHC.List-Functions
-- # import CoverPrelude

{------------------------------------------------------------------------
    Models
 ------------------------------------------------------------------------}

-- Expr -------------------------------------------------------------------

data Expr : Set where
  Val : Int -> Expr
  Add : Expr -> Expr -> Expr

modelExpr : Model Expr "E0:Expr"
modelExpr (Val n) x	= Exists (\n' -> x == "E0.Val" n' /\ modelInt n n')
modelExpr (Add e1 e2) x	= Exists (\e1' ->
			  Exists (\e2' ->
			    x == "E0.Add" e1' e2'
			    /\ modelExpr e1 e1'
			    /\ modelExpr e2 e2'
			  ))

-- Op ---------------------------------------------------------------------

data Op : Set where
  PUSH	: Int -> Op
  ADD	: Op

modelOp : Model Op "E0:Op"
modelOp (PUSH n) op = Exists (\n' -> op == "E0.PUSH" n' /\ modelInt n n')
modelOp ADD op	    = op == "E0.ADD"

"modelOp Injective" : Injective modelOp
"modelOp Injective" (PUSH n) op op' = fol-plugin("modelInt Injective")
"modelOp Injective" ADD op op'	    = fol-plugin()

Code  = List Op

modelCode = modelList modelOp

Stack = List Int
modelStack = modelList modelInt

-- eval -------------------------------------------------------------------

eval : Expr -> Int
eval (Val n)	= n
eval (Add x y)	= eval x + eval y

"eval model" : (modelExpr |-> modelInt) eval "E0.eval"
"eval model" (Val n)	= cover-plugin( include "E0" )
"eval model" (Add x y)	= cover-plugin( include "E0"
				      , include "GHC.Num"
				      , "eval model" x
				      , "eval model" y
				      )

-- comp -------------------------------------------------------------------

comp : Expr -> Code
comp (Val n)	= [PUSH n]
comp (Add x y)	= comp x ++ comp y ++ [ADD]

"comp model" : (modelExpr |-> modelCode) comp "E0.comp"
"comp model" (Val n)	= cover-plugin( include "E0" )
"comp model" (Add x y)	= cover-plugin( include "E0"
				      , "comp model" x
				      , "comp model" y
				      , "++ model" modelOp
				      , rem
				      , freeze modelList modelOp
				      )
  where
    rem : modelList modelOp [ADD] ("GHC.Base.:" "E0.ADD" "GHC.Base.[]")
    rem = fol-plugin()

-- exec -------------------------------------------------------------------

exec : Stack -> Code -> Partial Stack
exec s []		      = val s
exec s (PUSH n :: ops)	      = exec (n :: s) ops
exec (m :: n :: s) (ADD :: ops)  = exec (n + m :: s) ops
exec [n] (ADD :: ops)	      = bot
exec [] (ADD :: ops)	      = bot

"exec model" : (modelStack |-> modelCode |-> modelPartial modelStack) exec "E0.exec"
"exec model" s s' m_ss' [] ops' m_ops	= cover-plugin( include "E0", m_ss', m_ops )
"exec model" s s' m_ss' (PUSH n :: ops) ops' m_ops =
    cover-plugin( include "E0"
		, "exec model" (n :: s) ("GHC.Base.:" n s') fol-plugin(m_ss') ops
		, m_ops
		)
"exec model" (m :: n :: s) s' m_ss' (ADD :: ops) ops' m_ops =
    cover-plugin( include "E0"
		, include "GHC.Num"
		, include "GHC.List"
		, "exec model" (n + m :: s) ("GHC.Base.:" (n + m) s'') fol-plugin(rem_s)
			       ops ops'' rem_ops
		, rem1, rem2
		)
    where
      ops'' = "GHC.List.tail" ops'
      s''   = "GHC.List.tail" ("GHC.List.tail" s')

      rem_s : modelStack s s''
      rem_s = cover-plugin(include "GHC.List", m_ss')

      rem_ops : modelCode ops ops''
      rem_ops = cover-plugin(include "GHC.List", m_ops)

      rem1 : "GHC.Base.:" m ("GHC.Base.:" n s'') == s'
      rem1 = fol-plugin("modelList Injective" modelInt "modelInt Injective", rem_s, m_ss')

      rem2 : "GHC.Base.:" "E0.ADD" ops'' == ops'
      rem2 = fol-plugin("modelList Injective" modelOp "modelOp Injective", rem_ops, m_ops)

"exec model" [n] s' m_ss' (ADD :: ops) ops' m_ops  = cover-plugin(include "E0", m_ss', m_ops)
"exec model" [] s' m_ss' (ADD :: ops) ops' m_ops	  = cover-plugin(include "E0", m_ss', m_ops)

{-

exec (m :: n :: s) (ADD :: ops)	    ==
exec (n + m :: s) ops		    ~>
"exec" (n + m "::" "tail" ("tail" s')) ("tail" ops') ==
"exec" (m "::" n "::" "tail" ("tail" s')) ("ADD" :: "tail" ops')

-}

{-
{------------------------------------------------------------------------
    Proof
 ------------------------------------------------------------------------}

thm : (e : Expr) -> (s : Stack) -> (ops : Code) ->
       exec s (comp e ++ ops) == exec (eval e :: s) ops
thm (Val n) s ops     = fol-plugin()
thm (Add e1 e2) s ops = fol-plugin( thm e1, thm e2
				  , "++ assoc"
				  , (++)
				  , exec
				  )

cor : (e : Expr) -> (s : Stack) -> (ops : Code) ->
       let e'	= injExpr e
	   s'	= injStack s
	   ops'	= injCode ops
       in
	  "E0.exec" s' ("GHC.Base.++" ("E0.comp" e') ops') ==
	  "E0.exec" ("GHC.Base.:" ("E0.eval" e') s') ops'
cor e s ops = fol-plugin( thm
			, "eval model"
			, "comp model"
			, "exec model"
			, "++ model"
			, injList, injInt
			)
-}
