{-# OPTIONS -cpp -fglasgow-exts #-}

{-| Contains Data.Generics instances that are lacking in ghc-6.2, and
    some generic utility functions.
-}
module Utils.Generics where

import Data.Generics

isString :: GenericQ Bool
isString = mkQ False (const True :: String -> Bool)

everythingBut :: (r -> r -> r) -> GenericQ Bool -> GenericQ r -> GenericQ r
everythingBut (+) stop collect x
    | stop x	= collect x
    | otherwise	= foldr1 (+) $
		    collect x : gmapQ (everythingBut (+) stop collect) x

-- | Same as everywhereBut except that when the stop condition becomes
--   true, the function is called on the top level term (but not on the
--   children).
everywhereBut' :: GenericQ Bool -> GenericT -> GenericT
everywhereBut' q f x
    | q x       = f x
    | otherwise = f (gmapT (everywhereBut' q f) x)



-- Missing instances ------------------------------------------------------

#if __GLASGOW_HASKELL__ < 603

instance Data () where
    toConstr _		= mkConstr 1 "()" Prefix
    fromConstr _	= ()
    dataTypeOf _	= mkDataType [ mkConstr 1 "()" Prefix ]
    gfoldl f z ()	= z ()

instance (Data a, Data b, Data c) => Data (a,b,c) where
    toConstr _		= mkConstr 1 "(,,)" Infix
    fromConstr _	= (undefined, undefined, undefined)
    dataTypeOf _	= mkDataType [ mkConstr 1 "(,,)" Infix ]
    gfoldl f z (a,b,c)	= z (,,) `f` a `f` b `f` c

#endif

