
module TypeChecker.Monad.Definitions
    ( getDefinitions
    , lookupDef
    , addDefinition
    , delDefinition
    , addDefinitions
    , setDefinitions
    )
    where

import Prelude hiding	    ( print, putStr, putStrLn )
import IO.Output

import Control.Monad.State

import Syntax.Internal
import Syntax.Values
import Syntax.Position

import TypeChecker.Monad
import TypeChecker.Monad.Errors
import TypeChecker.Monad.Context
import TypeChecker.Monad.Options
import TypeChecker.Errors
import TypeChecker.Context
import TypeChecker.Abstractions
import TypeChecker.Values

import qualified Utils.Map as Map
import Utils.Debug
import Utils.Monads

-- Definitions ------------------------------------------------------------

getDefinitions :: TC Definitions
getDefinitions = liftM tcDefinitions get

-- Note: don't export
updateDefinitions f s = s { tcDefinitions = f $ tcDefinitions s }

lookupDef :: Name -> TC DefValue
lookupDef x =
    do	delta <- getDefinitions
	case Map.lookup x delta of
	    Just d  -> liftDefVal d
	    _	    -> failure $ notDefinedInternalError x
    where
	liftDefVal d@(DefVal h ar fv pa cs t) =
	    do	gamma <- getContext
		let t' = raiseByFrom (nofBoundVars gamma - fv) pa t
		return $ DefVal h ar fv pa cs t'

addDefinition :: Name -> DefValue -> TC ()
addDefinition x d =
    do	d' <- lambdaLift d
	verbose 10 $ liftIO $ putStrLn $ "addDefinition " ++ show x ++ " = " ++ show d'
	modify (updateDefinitions $ Map.insert x d')

delDefinition :: Name -> TC ()
delDefinition x =
    do	delta <- getDefinitions
	setDefinitions $ Map.delete x delta

lambdaLift :: DefValue -> TC DefValue
lambdaLift (DefVal h ar fv pa cs t) =
    do	tel <- reverse <$> getBoundContext
	let tel' = take fv tel
	    --t'	 = piAbstraction (map snd tel') t
	    cs'	 = mapClauses (map (liftClause $ map fst tel')) cs
	return $ DefVal h ar fv pa cs' t    -- Note: We keep the original type!
    where
	liftClause xs (ClauseVal ps v) = ClauseVal (replicate n VIdP ++ ps) v
	    where
		n = length xs

addDefinitions :: [(Name, DefValue)] -> TC ()
addDefinitions ds = mapM_ (uncurry addDefinition) ds

setDefinitions :: Definitions -> TC ()
setDefinitions defs = modify (updateDefinitions $ const defs)

